@@ -3,6 +3,7 @@
 """blender-downloader"""
 
 import argparse
+import json
 import math
 import os
 import re
@@ -14,20 +15,20 @@ from urllib.request import Request, urlopen, urlsplit
 
 from appdirs import user_data_dir
 from diskcache import Cache, Timeout as CacheTimeout
-from pkg_resources import parse_version
 from pkg_resources.extern.packaging.version import Version
 from tqdm import tqdm
 
 
 __author__ = "mondeja"
-__description__ = "Multiplatorm Blender portable release downloader script."
+__description__ = "Multiplatform Blender portable release downloader script."
 __title__ = "blender-downloader"
-__version__ = "0.0.11"
+__version__ = "0.0.19"
 
 QUIET = False
 
 SCRIPT_NEW_ISSUE_URL = f"https://github.com/{__author__}/{__title__}/issues/new"
-MINIMUM_VERSION_SUPPPORTED = "2.64"
+BLENDER_MANUAL_VERSIONS_URL = "https://docs.blender.org/PROD/versions.json"
+MINIMUM_VERSION_SUPPPORTED = "2.57"
 SUPPORTED_FILETYPES_EXTRACTION = [".bz2", ".gz", ".xz", ".zip", ".dmg"]
 NIGHLY_RELEASES_CACHE_EXPIRATION = 60 * 60 * 24  # 1 day
 CACHE = Cache(
@@ -35,6 +36,10 @@ CACHE = Cache(
 )
 
 
+class BlenderVersionNotFound(RuntimeError):
+    pass
+
+
 def get_running_os():
     if sys.platform == "darwin":
         return "macos"
@@ -73,9 +78,11 @@ def build_parser():
         "blender_version",
         nargs="?",
         metavar="BLENDER_VERSION",
-        help="Blender version to download. Could be a version number,"
-        " or the word 'stable' to download the current stable version."
-        f" The minium version supported is {MINIMUM_VERSION_SUPPPORTED}.",
+        help="Blender version to download. Could be a version number"
+        " or one of the words 'stable' (current stable version), 'lts'"
+        " (latest long term support version) and 'nightly' or 'daily'"
+        " (latest development release)."
+        f" The minimum version supported is {MINIMUM_VERSION_SUPPPORTED}.",
     )
     parser.add_argument(
         "-d",
@@ -92,7 +99,8 @@ def build_parser():
         action="store_true",
         help="Extract the content of the zipped release file. If this option"
         " is passed, the content of the release file will be extracted"
-        " in the same repository as '--output-directory' value argument.",
+        " in the same repository as '--output-directory' value argument."
+        " This is not supported for MacOS releases using Python3.6.",
     )
     parser.add_argument(
         "--remove-compressed",
@@ -131,10 +139,18 @@ def build_parser():
     parser.add_argument(
         "--bits",
         dest="bits",
-        default=64 if sys.maxsize > 2 ** 32 else 32,
+        default=64 if sys.maxsize > 2**32 else 32,
         type=int,
         help="Operative system bits. Keep in mind that Blender v2.80 was the"
-        " latest release with support operative systems wit 32 bits.",
+        " latest release with support operative systems with 32 bits.",
+    )
+    parser.add_argument(
+        "--arch",
+        dest="arch",
+        default=None,  # depending on availability between versions
+        type=str,
+        help="Architecture of the build. For most versions you don't need to"
+        " specify this parameter.",
     )
     parser.add_argument(
         "-l",
@@ -165,6 +181,17 @@ def build_parser():
     return parser
 
 
+def normalize_version(version):
+    if version.count(".") == 0:
+        version = f"{version.rstrip('.')}.0.0"
+    elif version.count(".") == 1:
+        version = f"{version.rstrip('.')}.0"
+    elif version.count(".") > 2:
+        version_split = version.split(".")
+        version = ".".join(version_split[i] for i in range(2))
+    return version.lstrip("v")
+
+
 def parse_args(args):
     parser = build_parser()
     if "-h" in args or "--help" in args:
@@ -175,7 +202,8 @@ def parse_args(args):
     # operative system by function and assert that is valid
     if hasattr(opts.operative_system, "__call__"):
         opts.operative_system = opts.operative_system()
-    if opts.operative_system not in ["linux", "macos", "windows"]:
+    opts.operative_system = opts.operative_system.lower()
+    if opts.operative_system not in {"linux", "macos", "windows"}:
         sys.stderr.write(
             f"Invalid operative system '{opts.operative_system}'. Must be"
             " either 'linux', 'macos' or 'windows'.\n"
@@ -193,11 +221,16 @@ def parse_args(args):
         if opts.blender_version is None:
             parser.print_help()
             sys.exit(1)
-        if opts.blender_version == "stable":
-            opts.blender_version = get_stable_release_version_number(
+        else:
+            opts.blender_version = opts.blender_version.lower()
+
+        if opts.blender_version in {"stable", "lts", "nightly", "daily"}:
+            opts.blender_version = discover_version_number_by_identifier(
+                opts.blender_version,
                 use_cache=opts.use_cache,
             )
-        opts.blender_version = opts.blender_version.lstrip("v")
+        elif Version(opts.blender_version) >= Version("2.83"):
+            opts.blender_version = normalize_version(opts.blender_version)
     else:
         opts.blender_version = None
 
@@ -218,10 +251,7 @@ def parse_args(args):
         sys.exit(1)
 
     # assert compatible bits
-    if opts.bits == 32 and (
-        opts.blender_version == "stable"
-        or parse_version(opts.blender_version) > Version("2.80")
-    ):
+    if opts.bits == 32 and Version(opts.blender_version) > Version("2.80"):
         sys.stderr.write(
             "The latest Blender version with support for 32 bits systems is"
             " v2.80. Please, specify a more recent version of Blender.\n"
@@ -237,32 +267,58 @@ def parse_args(args):
     return opts
 
 
-def get_stable_release_version_number(use_cache=True):
-    """Retrieves the latest Blender stable release version number from their
-    website.
+def discover_version_number_by_identifier(identifier, use_cache=True):
+    """Discover a Blender version number given an identifier.
 
     Parameters
     ----------
 
+    identifier : str
+      Version identifier. Can be either 'stable', 'lts' or 'nightly'.
     use_cache : bool
-      Use cache requesting Blender repositories.
+      Use cache requesting Blender versions from manual.
     """
     res = GET(
-        "https://www.blender.org/download/",
+        BLENDER_MANUAL_VERSIONS_URL,
         expire=NIGHLY_RELEASES_CACHE_EXPIRATION,
         use_cache=use_cache,
     )
-    try:
-        return re.search(r"blender-(\d+\.\d+\.\d+)-", res).group(1)
-    except AttributeError as err:
-        if "'NoneType' object has no attribute 'group'" in str(err):
+    versions_json = json.loads(res)
+
+    if identifier == "stable":
+        latest_Version = None
+        for minor_version, version_data in versions_json.items():
+            if "dev" in version_data:
+                continue
+
+            minor_Version = Version(minor_version)
+            if latest_Version is None or minor_Version > latest_Version:
+                latest_Version = minor_Version
+
+        if latest_Version is None:
             sys.stderr.write(
-                "Failed to obtain the stable release version of Blender.\n"
-                " Please, report this issue using the next URL:"
-                f" {SCRIPT_NEW_ISSUE_URL}\n"
+                "Error trying to retrieve the stable release from Blender"
+                " repositories. Please, submit an issue to"
+                " {SCRIPT_NEW_ISSUE_URL}.\n"
             )
             sys.exit(1)
-        raise err
+    elif identifier in {"lts", "nightly", "daily"}:
+        expected_substr_in_data = "lts" if identifier == "lts" else "dev"
+        latest_Version = None
+
+        for minor_version, version_data in versions_json.items():
+            if expected_substr_in_data not in version_data.lower():
+                continue
+            minor_Version = Version(minor_version)
+            if latest_Version is None or minor_Version > latest_Version:
+                latest_Version = minor_Version
+    else:
+        sys.stderr.write(
+            f"Invalid identifier '{identifier}' for Blender version. Possible"
+            " values are 'stable', 'lts' and 'nightly'.\n"
+        )
+        sys.exit(1)
+    return normalize_version(str(latest_Version))
 
 
 def _build_download_repo_expected_os_identifier(
@@ -270,7 +326,9 @@ def _build_download_repo_expected_os_identifier(
     major_minor_blender_Version,
 ):
     if operative_system == "macos":
-        if major_minor_blender_Version < Version("2.65"):
+        if major_minor_blender_Version < Version("2.61"):
+            expected_os_identifier = "OSX"
+        elif major_minor_blender_Version < Version("2.65"):
             expected_os_identifier = "release-OSX"
         elif major_minor_blender_Version < Version("2.79"):
             # previous to v2.79, macOS was identified by "OSX"
@@ -278,7 +336,9 @@ def _build_download_repo_expected_os_identifier(
         else:
             expected_os_identifier = "mac"  # some macOS, other macos
     elif operative_system == "windows":
-        if major_minor_blender_Version < Version("2.66"):
+        if major_minor_blender_Version < Version("2.61"):
+            expected_os_identifier = "windows"
+        elif major_minor_blender_Version < Version("2.66"):
             expected_os_identifier = "release-windows"
         else:
             expected_os_identifier = operative_system
@@ -290,6 +350,7 @@ def _build_download_repo_expected_os_identifier(
 def _build_download_repo_release_file_validator(
     operative_system,
     bits,
+    arch,
     major_minor_blender_Version,
 ):
     if operative_system == "windows":
@@ -317,7 +378,7 @@ def _build_download_repo_release_file_validator(
                 if not filename.endswith(compressed_ext):
                     return False
             else:
-                bits_id = "x86_64" if bits == 64 else "i686"
+                bits_id = "x86_64" if (bits == 64 or arch == "x86_64") else "i686"
                 if not filename.endswith(f"{bits_id}{compressed_ext}"):
                     return False
             return True
@@ -334,13 +395,17 @@ def _build_download_repo_release_file_validator(
         def valid_release_file(filename):
             if major_minor_blender_Version < Version("2.72"):
                 # previous to v2.72, macos release supported 32 bits
-                bits_id = "x86_64" if bits == 64 else "i386"
+                bits_id = "x86_64" if (bits == 64 or arch == "x86_64") else "i386"
                 if not filename.endswith(f"{bits_id}{compressed_ext}"):
                     return False
             elif major_minor_blender_Version >= Version("2.93"):
                 # from v2.93, Blender supports arm64 builds for macOS
-                if not filename.endswith(f"x64{compressed_ext}"):
-                    return False
+                if arch in ["x64", "arm64"]:
+                    if not filename.endswith(f"{arch}{compressed_ext}"):
+                        return False
+                else:
+                    if not filename.endswith(f"x64{compressed_ext}"):
+                        return False
             else:
                 if not filename.endswith(compressed_ext):
                     return False
@@ -350,9 +415,9 @@ def _build_download_repo_release_file_validator(
 
 
 def get_legacy_release_download_url(
-    blender_version, operative_system, bits, use_cache=True
+    blender_version, operative_system, bits, arch, use_cache=True
 ):
-    """Retrieves the download URL for a specifc release version of Blender.
+    """Retrieves the download URL for a specific legacy release of Blender.
 
     Parameters
     ----------
@@ -367,6 +432,9 @@ def get_legacy_release_download_url(
     bits : str
       Number of bits of the system for the release.
 
+    arch : str
+      Identifier of the architecture for which the release will be retrieved.
+
     use_cache : bool
       Use cache requesting Blender repositories.
     """
@@ -384,15 +452,6 @@ def get_legacy_release_download_url(
 
     url = "https://download.blender.org/release/"
 
-    version_not_found_error_message = lambda: (
-        f"The release '{blender_version}' can't be located in official"
-        " Blender repositories.\nMake sure that you are passing a valid"
-        f" version.\nYou can check all valid releases at: {url}\n\n"
-        f"If you think that '{blender_version}' is a valid release and"
-        " this is a problem with the downloader,\nplease, report it to"
-        f" {SCRIPT_NEW_ISSUE_URL}\n"
-    )
-
     res = GET(url, use_cache=use_cache)
 
     version_path = f"Blender{major_minor_blender_version}/"
@@ -403,8 +462,7 @@ def get_legacy_release_download_url(
             break
 
     if not _version_path_found:
-        sys.stderr.write(version_not_found_error_message())
-        sys.exit(1)
+        raise BlenderVersionNotFound()
 
     major_minor_blender_release_url = f"{url}{version_path}"
 
@@ -421,6 +479,7 @@ def get_legacy_release_download_url(
     valid_release_file = _build_download_repo_release_file_validator(
         operative_system,
         bits,
+        arch,
         major_minor_blender_Version,
     )
 
@@ -439,8 +498,70 @@ def get_legacy_release_download_url(
         break
 
     if download_url is None:
-        sys.stderr.write(version_not_found_error_message())
-        sys.exit(1)
+        raise BlenderVersionNotFound()
+
+    return download_url
+
+
+def get_nightly_release_download_url(
+    blender_version, operative_system, arch, use_cache=True
+):
+    """Retrieves the download URL for a specific nightly release of Blender.
+
+    Parameters
+    ----------
+
+    blender_version : str
+      Version for which the URL will be discovered. Should be a valid version
+      of Blender, otherwise shows an error an the script will exit with 1 code.
+
+    operative_system : str
+      Operative system correspondent to the release.
+
+    bits : str
+      Number of bits of the system for the release.
+
+    arch : str
+      Identifier of the architecture for which the release will be retrieved.
+
+    use_cache : bool
+      Use cache requesting Blender repositories.
+    """
+    url = "https://builder.blender.org/download/daily/archive/"
+    res = GET(url, use_cache=use_cache)
+    urls = re.findall(rf'"({url}[^"]+)"', res)
+
+    expected_os_identifier = (
+        "darwin" if operative_system == "macos" else operative_system
+    )
+    expected_extension = {
+        "linux": "tar.xz",
+        "windows": "zip",
+        "macos": "dmg",
+    }[operative_system]
+
+    download_url = None
+
+    for url in urls:
+        if expected_os_identifier not in url:
+            continue
+        if f"/blender-{blender_version}-" not in url:
+            continue
+
+        arch_ext_split = url.split(f"{expected_os_identifier}.")[1].split("-")
+        version_ext = ".".join(arch_ext_split[-1].split(".")[1:])
+        if version_ext != expected_extension:
+            continue
+
+        version_arch = arch_ext_split[0]
+        if arch is not None and arch != version_arch:
+            continue
+
+        download_url = url
+        break
+
+    if download_url is None:
+        raise BlenderVersionNotFound()
 
     return download_url
 
@@ -570,7 +691,7 @@ def extract_release(zipped_filepath, quiet=False):
         running_os = get_running_os()
         if running_os != "macos":
             sys.stderr.write(
-                "blender-downloader can't mount MacOSX '.dmg' image files like"
+                "blender-downloader can't mount MacOS '.dmg' image files like"
                 f" '{extracted_directory_filepath}' in"
                 f" {running_os.capitalize()}, so you should install Blender"
                 " manually.\n"
@@ -585,7 +706,7 @@ def extract_release(zipped_filepath, quiet=False):
 
         with dmglib.attachedDiskImage(zipped_filepath) as mounted_dmg:
             contents_parent_dirpath = None
-            for dirpath, dirnames, files in os.walk(mounted_dmg[0]):
+            for dirpath, _, _ in os.walk(mounted_dmg[0]):
                 if (
                     os.path.basename(dirpath) == "Contents"
                     and "blender.app" in dirpath.lower()
@@ -706,7 +827,7 @@ def print_executables(
             python_executable_filepath
         ):
             sys.stderr.write(
-                "Builtin Blender Python intepreter executable filepath not found\n"
+                "Builtin Blender Python interpreter executable filepath not found\n"
             )
             error = True
         else:
@@ -717,7 +838,7 @@ def print_executables(
 
 
 def list_available_blender_versions(
-    maximum_versions, operative_system, bits, use_cache=True
+    maximum_versions, operative_system, bits, arch, use_cache=True
 ):
     """Prints to stdout all Blender versions available in official repositories.
 
@@ -740,15 +861,31 @@ def list_available_blender_versions(
     use_cache : bool
       Use cache requesting Blender repositories.
     """
-    n_versions, versions_found = (1, [])
+    n_versions, versions_found = (0, [])
 
-    # Stable version number
-    stable_version = get_stable_release_version_number(use_cache=use_cache)
-    sys.stdout.write(f"{stable_version}\n")
-    versions_found.append(stable_version)
+    # Nightly version number
+    nightly_version = discover_version_number_by_identifier(
+        "nightly",
+        use_cache=use_cache,
+    )
+    sys.stdout.write(f"{nightly_version}\n")
+    versions_found.append(nightly_version)
+    n_versions += 1
     if maximum_versions < 2:
         return 0
 
+    # Stable version number
+    stable_version = discover_version_number_by_identifier(
+        "stable",
+        use_cache=use_cache,
+    )
+    versions_found.append(stable_version)
+    if maximum_versions < 3:
+        sys.stdout.write(f"{stable_version}\n")
+        return 0
+    stable_Version = Version(stable_version)
+    _stable_version_printed = False
+
     url = "https://download.blender.org/release/"
     response_lines = GET(url, use_cache=use_cache).splitlines()
     version_matcher = re.compile(r"^\d+\.\d+$")
@@ -780,6 +917,7 @@ def list_available_blender_versions(
         valid_release_file = _build_download_repo_release_file_validator(
             operative_system,
             bits,
+            arch,
             major_minor_blender_Version,
         )
 
@@ -808,9 +946,17 @@ def list_available_blender_versions(
                 continue
 
             # print version
+            #   print stable version in their correct place
+            if Version(version) < stable_Version and not _stable_version_printed:
+                n_versions += 1
+                sys.stdout.write(f"{stable_version}\n")
+                _stable_version_printed = True
+                if n_versions >= maximum_versions:
+                    break
+
             versions_found.append(version)
-            n_versions += 1
             sys.stdout.write(f"{version}\n")
+            n_versions += 1
 
             if n_versions >= maximum_versions:
                 break
@@ -830,7 +976,7 @@ def run(args=[]):
             CACHE.clear()
         except CacheTimeout:
             sys.stderr.write(
-                "An error ocurred clearing blender-downloader's cache.\n"
+                "An error happen clearing blender-downloader's cache.\n"
                 f"Please, submit an issue to {SCRIPT_NEW_ISSUE_URL} if the"
                 " problem persists.\n"
             )
@@ -844,15 +990,36 @@ def run(args=[]):
             opts.list,
             opts.operative_system,
             opts.bits,
+            opts.arch,
             use_cache=opts.use_cache,
         )
-
-    download_url = get_legacy_release_download_url(
-        opts.blender_version,
-        opts.operative_system,
-        opts.bits,
-        use_cache=opts.use_cache,
-    )
+    try:
+        download_url = get_legacy_release_download_url(
+            opts.blender_version,
+            opts.operative_system,
+            opts.bits,
+            opts.arch,
+            use_cache=opts.use_cache,
+        )
+    except BlenderVersionNotFound:
+        try:
+            download_url = get_nightly_release_download_url(
+                opts.blender_version,
+                opts.operative_system,
+                opts.arch,
+                use_cache=opts.use_cache,
+            )
+        except BlenderVersionNotFound:
+            version_not_found_error_message = (
+                f"The release '{opts.blender_version}' can't be located in official"
+                " Blender repositories.\nMake sure that you are passing a valid"
+                f" version.\n\n"
+                f"If you think that '{opts.blender_version}' is a valid release and"
+                " this is a problem with blender-downloader, please, report it to"
+                f" {SCRIPT_NEW_ISSUE_URL}\n"
+            )
+            sys.stderr.write(version_not_found_error_message)
+            sys.exit(1)
     downloaded_release_filepath = download_release(
         download_url,
         opts.output_directory,
